import ipkiss3.all as i3
from matplotlib.offsetbox import AnchoredText
import matplotlib.pyplot as plt


def visualize_crosstalk(channel, smatrix, peaks, near_crosstalk, far_crosstalk, bands):
    peaks_x = peaks[channel]["wavelength"]
    peak_powers = peaks[channel]["power"]
    peak_power = peak_powers[0]

    fig, (ax1, ax2) = plt.subplots(2, figsize=(15, 20))
    # use the following if you prefer a column layout:
    # fig, (ax1, ax2) = plt.subplots(1, 2)

    # visualise the near_crosstalk of a channel
    ax1.add_artist(
        AnchoredText(
            "near_crosstalk of channel '{}'".format(channel),
            frameon=True,
            loc="lower left",
        )
    )

    for channel in ["A", "B", "C", "D"]:
        ax1.plot(
            smatrix.sweep_parameter_values,
            i3.signal_power_dB(smatrix["A", channel]),
            label="Channel {}".format(channel),
        )

    ax1.legend(loc="lower right")

    ax1.axhline(y=peak_power + near_crosstalk[channel], color="darkgrey", linestyle=":")

    for left, right in bands[channel]:
        ax1.axvline(x=left, color="red", linestyle="--")
        ax1.axvline(x=right, color="red", linestyle="--")

    ax1.plot(
        bands[channel][0][0],
        peak_power + near_crosstalk[channel],
        "ro",
    )

    ax1.annotate(
        "",
        xy=tuple((peaks_x[0], peak_power + near_crosstalk[channel])),
        xycoords="data",
        xytext=tuple((peaks_x[0], peak_power)),
        textcoords="data",
        arrowprops={"arrowstyle": "<->"},
        color="black",
    )

    ax1.annotate(
        "crosstalk={:.2f}dB".format(near_crosstalk[channel]),
        xy=(peaks_x[0] + 0.0001, peak_power + near_crosstalk[channel] / 2.0),
        xycoords="data",
        textcoords="data",
    )

    # now visualise the far_crosstalk of a channel
    ax2.add_artist(
        AnchoredText(
            "far_crosstalk of channel '{}'".format(channel),
            frameon=True,
            loc="lower left",
        )
    )

    for channel in ["A", "B", "C", "D"]:
        ax2.plot(
            smatrix.sweep_parameter_values,
            i3.signal_power_dB(smatrix["A", channel]),
            label="Channel {}".format(channel),
        )

    ax2.axhline(y=peak_power + far_crosstalk[channel], color="darkgrey", linestyle=":")

    for left, right in bands[channel]:
        ax2.axvline(x=left, color="red", linestyle="--")
        ax2.axvline(x=right, color="red", linestyle="--")

    ax2.plot(
        bands[channel][0][0],
        peak_power + far_crosstalk[channel],
        "ro",
    )

    ax2.annotate(
        "",
        xy=tuple((peaks_x[0], peak_power + far_crosstalk[channel])),
        xycoords="data",
        xytext=tuple((peaks_x[0], peak_power)),
        textcoords="data",
        arrowprops={"arrowstyle": "<->"},
        color="black",
    )

    ax2.annotate(
        "crosstalk={:.2f}dB".format(far_crosstalk[channel]),
        xy=(peaks_x[0] + 0.0001, peak_power + far_crosstalk[channel] / 2.0),
        xycoords="data",
        textcoords="data",
    )
    plt.rc("font", size=15)
    plt.show()
